@@ -30,6 +30,10 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <unistd.h>
+#include <time.h>
+#ifdef HAVE_MALLINFO2
+#include <malloc.h>
+#endif
 #include <libgen.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -45,12 +49,14 @@
 #include "libaudit.h"
 #include "auplugin.h"
 #include "audispd-pconfig.h"
+#include "queue.h"
 
-#define DEFAULT_PATH "/var/run/audispd_events"
+#define DEFAULT_PATH AUDIT_RUN_DIR"/audispd_events"
+#define STATE_FILE AUDIT_RUN_DIR"/af_unix.state"
 //#define DEBUG
 
 /* Global Data */
-static volatile int stop = 0, hup = 0;
+static volatile int stop = 0, hup = 0, usr1 = 0;
 char rx_buf[MAX_AUDIT_EVENT_FRAME_SIZE+1];
 int sock = -1, conn = -1, client = 0;
 struct pollfd pfd[3];
@@ -59,6 +65,19 @@ format_t format = -1;
 char *path = NULL;
 int inbound_protocol = -1;
 
+#ifdef HAVE_MALLINFO2
+static struct mallinfo2 last_mi;
+#endif
+
+#define DEFAULT_QUEUE_DEPTH 512
+#define QUEUE_ENTRY_SIZE MAX_AUDIT_EVENT_FRAME_SIZE+1
+
+static size_t queue_depth = DEFAULT_QUEUE_DEPTH;
+static struct queue *queue;
+static const unsigned char *out_buf;
+static size_t out_len;
+static size_t out_off;
+
 /*
  * SIGTERM handler
  *
@@ -81,6 +100,50 @@ static void hup_handler(int sig)
 	hup = 1;
 }
 
+static void usr1_handler(int sig)
+{
+	usr1 = 1;
+}
+
+#ifdef HAVE_MALLINFO2
+static void write_memory_state(FILE *f)
+{
+	struct mallinfo2 mi = mallinfo2();
+
+	fprintf(f, "glibc arena (total memory) is: %zu KiB, was: %zu KiB\n",
+		(size_t)mi.arena/1024, (size_t)last_mi.arena/1024);
+	fprintf(f, "glibc uordblks (in use memory) is: %zu KiB, was: %zu KiB\n",
+		(size_t)mi.uordblks/1024,(size_t)last_mi.uordblks/1024);
+	fprintf(f,"glibc fordblks (total free space) is: %zu KiB, was: %zu KiB\n",
+		(size_t)mi.fordblks/1024,(size_t)last_mi.fordblks/1024);
+
+	memcpy(&last_mi, &mi, sizeof(struct mallinfo2));
+}
+#endif
+
+static void write_state_report(void)
+{
+	char buf[64];
+	mode_t u = umask(0137); // allow 0640
+	FILE *f = fopen(STATE_FILE, "w");
+	umask(u);
+	if (f == NULL)
+		return;
+
+	time_t now = time(NULL);
+	strftime(buf, sizeof(buf), "%x %X", localtime(&now));
+	fprintf(f, "current time = %s\n", buf);
+	fprintf(f, "client_connected = %s\n", client ? "yes" : "no");
+	fprintf(f, "queue_length = %zu\n", q_queue_length(queue));
+	fprintf(f, "max_queue_length = %zu\n", q_max_queue_length(queue));
+	fprintf(f, "queue_size = %zu\n", q_queue_size(queue));
+#ifdef HAVE_MALLINFO2
+	write_memory_state(f);
+#endif
+	fclose(f);
+}
+
+
 int create_af_unix_socket(const char *spath, int mode)
 {
 	struct sockaddr_un addr;
@@ -94,7 +157,7 @@ int create_af_unix_socket(const char *spath, int mode)
 		return -1;
 	}
 	setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
-			(char *)&one, sizeof (int));
+		   (char *)&one, sizeof (int));
 #ifdef DEBUG
 	printf("%o %s\n", mode, spath);
 #else
@@ -132,14 +195,25 @@ int setup_socket(int argc, char *argv[])
 	for (int i = 1; i < argc; i++) {
 		char *arg = argv[i];
 		if (isdigit((unsigned char)arg[0])) {
-			// parse mode
 			errno = 0;
-			mode = strtoul(arg, NULL, 8);
-			if (errno) {
-				syslog(LOG_ERR,
-					"Error converting %s (%s)",
-					argv[i], strerror(errno));
-				mode = 0;
+			if (mode == 0) {
+				// parse mode
+				mode = strtoul(arg, NULL, 8);
+				if (errno) {
+					syslog(LOG_ERR,
+						"Error converting %s (%s)",
+						argv[i], strerror(errno));
+					mode = 0;
+				}
+			} else {
+				// parse queue depth
+				queue_depth = strtoul(arg, NULL, 10);
+				if (errno || queue_depth == 0) {
+					syslog(LOG_ERR,
+						"Error converting %s (%s)",
+						argv[i], strerror(errno));
+					queue_depth = DEFAULT_QUEUE_DEPTH;
+				}
 			}
 		} else if (strchr(arg, '/') != NULL) {
 			// parse path
@@ -157,14 +231,14 @@ int setup_socket(int argc, char *argv[])
 					free(dir);
 				} else {
 					syslog(LOG_ERR,
-						"Couldn't open %s (%s)",
-						base, strerror(errno));
+					       "Couldn't open %s (%s)",
+					       base, strerror(errno));
 					free(dir);
 					exit(1);
 				}
 			} else {
 				syslog(LOG_ERR, "Malformed path %s",
-					path);
+				       path);
 				exit(1);
 			}
 		} else {
@@ -192,12 +266,14 @@ int setup_socket(int argc, char *argv[])
 			syslog(LOG_INFO, "Using default format");
 		}
 	}
+	if (queue_depth == DEFAULT_QUEUE_DEPTH)
+		syslog(LOG_INFO, "Using default queue depth");
 
 	return create_af_unix_socket(path, mode);
 }
 
 static int event_to_string(struct audit_dispatcher_header *hdr,
-				char *data, char **out, int *outlen)
+			   char *data, char **out, int *outlen)
 {
 	char *v = NULL, *ptr, unknown[32];
 	int len;
@@ -210,11 +286,11 @@ static int event_to_string(struct audit_dispatcher_header *hdr,
 			type = audit_msg_type_to_name(hdr->type);
 			if (type == NULL) {
 				snprintf(unknown, sizeof(unknown),
-					"UNKNOWN[%u]", hdr->type);
+					 "UNKNOWN[%u]", hdr->type);
 				type = unknown;
 			}
 			len = asprintf(&v, "type=%s msg=%.*s\n",
-					type, hdr->size, data);
+				       type, hdr->size, data);
 		} else if (inbound_protocol == F_BINARY &&
 			   hdr->ver == AUDISP_PROTOCOL_VER2) {
 			// Protocol 2 events are already formatted
@@ -257,7 +333,7 @@ static int event_to_string(struct audit_dispatcher_header *hdr,
  * 0 when EOF is reached.
  */
 static int read_binary_record(int fd, struct audit_dispatcher_header *hdr,
-                              char *data)
+			      char *data)
 {
 	size_t len = sizeof(*hdr);
 	char *ptr = (char *)hdr;
@@ -293,13 +369,46 @@ static int read_binary_record(int fd, struct audit_dispatcher_header *hdr,
 	return sizeof(*hdr) + hdr->size;
 }
 
+int failed_append = 0;
 void read_audit_record(int ifd)
 {
 	int len;
 
-	if (inbound_protocol == F_BINARY || inbound_protocol == -1) {
+	// If it's the first call, detect which inbound protocol we are using
+	if (inbound_protocol == -1) {
+		unsigned char peek[4];
+		ssize_t rc;
+
+		// audisp uses socketpair to setup stdin, use recvfrom to
+		// peek into what the protocol might be
+		rc = recvfrom(ifd, peek, sizeof(peek), MSG_PEEK, NULL, NULL);
+		if (rc < 0) {
+			if (errno == ENOTSOCK) {
+				// Let's assume it is a file which means string
+				inbound_protocol = F_STRING;
+				goto resolved;
+			}
+			syslog(LOG_ERR, "recvfrom failed (%s)",
+			       strerror(errno));
+			stop = 1;
+			return;
+		}
+		if (rc == 0) {
+			stop = 1;
+			return;
+		}
+		// If its string, there will be an ascii letter in the first
+		// character. If it's binary, it will be a 1 (0 is ancient)
+		if (peek[0] == 1 || peek[0] == 0)
+			inbound_protocol = F_BINARY;
+		else
+			inbound_protocol = F_STRING;
+	}
+
+resolved:
+	if (inbound_protocol == F_BINARY) {
 		struct audit_dispatcher_header *hdr =
-				(struct audit_dispatcher_header *)rx_buf;
+			(struct audit_dispatcher_header *)rx_buf;
 		char *data = rx_buf + sizeof(*hdr);
 
 		len = read_binary_record(ifd, hdr, data);
@@ -308,98 +417,83 @@ void read_audit_record(int ifd)
 				stop = 1;
 			return;
 		}
-		if (inbound_protocol == -1)
-			inbound_protocol = F_BINARY;
 
 		if (client && !stop) {
-			int rc;
-
 			if (format == F_STRING) {
 				char *str = NULL;
 				int str_len = 0;
 
 				if (event_to_string(hdr, data, &str,
-							&str_len) < 0)
+						    &str_len) < 0)
 					return;
 
-				do {
-					rc = write(conn, str, str_len);
-				} while (rc < 0 && errno == EINTR);
+				if (q_append(queue, str, str_len) != 0) {
+					if (failed_append < 5)
+						syslog(LOG_ERR,
+				  "Dropping event - queue append failed (%s)",
+						       strerror(errno));
+					failed_append++;
+				} else
+					failed_append = 0;
 				free(str);
 			} else if (format == F_BINARY) {
-				struct iovec vec[2];
-
-				vec[0].iov_base = hdr;
-				vec[0].iov_len = sizeof(*hdr);
-
-				vec[1].iov_base = data;
-				vec[1].iov_len = hdr->size;
-
-				do {
-					rc = writev(conn, vec, 2);
-				} while (rc < 0 && errno == EINTR);
-				if (rc < 0 && errno == EPIPE) {
-					close(conn);
-					conn = -1; // FIXME: is this right?
-					client = 0;
-					auplugin_fgets_clear();
+				int total = sizeof(*hdr) + hdr->size;
+				char *buf = malloc(total);
+				if (buf) {
+					memcpy(buf, hdr, sizeof(*hdr));
+					memcpy(buf + sizeof(*hdr), data,
+					       hdr->size);
+					if (q_append(queue, buf, total) != 0) {
+						if (failed_append < 5)
+							syslog(LOG_ERR,
+				  "Dropping event - queue append failed (%s)",
+							   strerror(errno));
+						failed_append++;
+					} else
+						failed_append = 0;
+					free(buf);
 				}
 			}
 		}
 	} else {
 		do {
 			len = auplugin_fgets(rx_buf,
-					MAX_AUDIT_EVENT_FRAME_SIZE + 1, ifd);
+					   MAX_AUDIT_EVENT_FRAME_SIZE + 1, ifd);
 			if (len > 0) {
-				if (inbound_protocol == -1)
-					inbound_protocol = F_STRING;
 				if (client && !stop) {
-					int rc;
-					char *data = rx_buf +
-					 sizeof(struct audit_dispatcher_header);
-					struct audit_dispatcher_header *hdr =
-					 (struct audit_dispatcher_header *)rx_buf;
-
 					if (format == F_STRING) {
-						char *str = NULL;
-						int str_len = 0;
-						if (event_to_string(hdr, data,
-								    &str,
-								&str_len) < 0)
-							continue;
-
-						do {
-							rc = write(conn, str,
-								   str_len);
-						} while (rc < 0 && errno == EINTR);
-						free(str);
+						if (q_append(queue, rx_buf,
+							     len) != 0)
+							syslog(LOG_ERR,
+						    "Queue append failed (%s)",
+							       strerror(errno));
 					} else if (format == F_BINARY) {
-						struct iovec vec[2];
+						struct audit_dispatcher_header hdr;
 
-						vec[0].iov_base = hdr;
-						vec[0].iov_len =
-								sizeof(*hdr);
-
-						vec[1].iov_base = data;
-						vec[1].iov_len =
-						    MAX_AUDIT_MESSAGE_LENGTH;
-
-						do {
-							rc = writev(conn, vec,
-									2);
-						} while (rc < 0 && errno == EINTR);
-						if (rc < 0 && errno == EPIPE) {
-							close(conn);
-							conn = -1;
-							client = 0;
-							auplugin_fgets_clear();
+						hdr.ver = AUDISP_PROTOCOL_VER2;
+						hdr.hlen = sizeof(struct audit_dispatcher_header);
+						hdr.type = 0;
+						hdr.size = len;
+						int total = sizeof(hdr) + len;
+						char *buf = malloc(total);
+						if (buf) {
+							memcpy(buf, &hdr,
+							       sizeof(hdr));
+							memcpy(buf+sizeof(hdr),
+							       rx_buf, len);
+							if (q_append(queue, buf,
+								    total) != 0)
+								syslog(LOG_ERR,
+						     "Queue append failed (%s)",
+							       strerror(errno));
+							free(buf);
 						}
 					}
 				}
 			} else if (auplugin_fgets_eof())
 				stop = 1;
 		} while (!stop &&
-			auplugin_fgets_more(MAX_AUDIT_EVENT_FRAME_SIZE));
+			 auplugin_fgets_more(MAX_AUDIT_EVENT_FRAME_SIZE));
 	}
 }
 
@@ -421,21 +515,72 @@ void accept_connection(void)
 	}
 }
 
+static void send_queue(void)
+{
+	ssize_t rc;
+
+	while (!q_empty(queue) && client && !stop) {
+		if (out_off == 0) {
+			int r = q_peek(queue, &out_buf, &out_len);
+			if (r <= 0) {
+				if (r < 0)
+					syslog(LOG_ERR,
+					       "Queue peek failed (%s)",
+					       strerror(errno));
+				return;
+			}
+		}
+
+		do {
+			rc = write(conn, out_buf + out_off,
+				   out_len - out_off);
+		} while (rc < 0 && errno == EINTR);
+		if (rc < 0) {
+			if (errno == EAGAIN)
+				return;
+			if (errno == EPIPE) {
+				close(conn);
+				conn = -1;
+				client = 0;
+				auplugin_fgets_clear();
+				out_off = 0;
+				out_len = 0;
+				out_buf = NULL;
+			}
+			return;
+		}
+		out_off += rc;
+		if (out_off == out_len) {
+			q_drop_head(queue);
+			out_off = 0;
+			out_len = 0;
+			out_buf = NULL;
+		}
+	}
+}
+
 void event_loop(int ifd)
 {
 	// setup poll
 	pfd[0].fd = ifd;	//stdin
 	pfd[0].events = POLLIN;
 	pfd[1].fd = sock;	// listen socket
-	pfd[1].events = POLLIN|POLLOUT;
+	pfd[1].events = POLLIN;
 
 	// loop on poll until stop - not doing HUP for now
 	while (!stop) {
 		int rc;
 
+		if (usr1) {
+			write_state_report();
+			usr1 = 0;
+		}
+
 		if (client) {
-			pfd[2].fd = conn;	// the client
+			pfd[2].fd = conn;       // the client
 			pfd[2].events = POLLHUP;
+			if (!q_empty(queue))
+				pfd[2].events |= POLLOUT;
 		}
 
 		rc = poll(pfd, 2 + client, -1);
@@ -458,6 +603,9 @@ void event_loop(int ifd)
 				conn = -1;
 				client = 0;
 				auplugin_fgets_clear();
+				out_off = 0;
+				out_len = 0;
+				out_buf = NULL;
 			}
 			// auditd closed it's socket, exit
 			if (pfd[0].revents & POLLHUP) {
@@ -470,10 +618,14 @@ void event_loop(int ifd)
 				read_audit_record(ifd);
 			}
 
-			if (pfd[1].revents & (POLLIN|POLLOUT)) {
+			if (pfd[1].revents & POLLIN) {
 				// someone connected, accept it
 				accept_connection();
+				send_queue();
 			}
+
+			if (client && (pfd[2].revents & POLLOUT))
+				send_queue();
 		}
 	}
 	if (stop == 1)
@@ -497,7 +649,10 @@ int main(int argc, char *argv[])
 
 	/* Set handler for the ones we care about */
 	sa.sa_handler = hup_handler;
+	sa.sa_flags = 0;
 	sigaction(SIGHUP, &sa, NULL);
+	sa.sa_handler = usr1_handler;
+	sigaction(SIGUSR1, &sa, NULL);
 	sa.sa_sigaction = term_handler;
 	sa.sa_flags = SA_SIGINFO;
 	sigaction(SIGTERM, &sa, NULL);
@@ -511,7 +666,8 @@ int main(int argc, char *argv[])
 
 	// Initialize the socket
 	if (setup_socket(argc, argv)) {
-		syslog(LOG_ERR,"audisp-af_unix plugin exiting due to errors setting up socket");
+		syslog(LOG_ERR,"audisp-af_unix plugin exiting due to errors "
+		       "setting up socket");
 		exit(1);
 	}
 
@@ -519,8 +675,15 @@ int main(int argc, char *argv[])
 	// Drop capabilities
 	capng_clear(CAPNG_SELECT_BOTH);
 	if (capng_apply(CAPNG_SELECT_BOTH))
-		syslog(LOG_WARNING, "audisp-af_unix plugin was unable to drop capabilities, continuing with elevated priviles");
+		syslog(LOG_WARNING, "audisp-af_unix plugin was unable to "
+		       "drop capabilities, continuing with elevated priviles");
 #endif
+	queue = q_open(queue_depth, QUEUE_ENTRY_SIZE);
+	if (queue == NULL) {
+		syslog(LOG_ERR, "Unable to create queue (%s)",
+		       strerror(errno));
+		exit(1);
+	}
 	syslog(LOG_INFO, "audisp-af_unix plugin is listening for events");
 	event_loop(ifd);
 
@@ -529,10 +692,10 @@ int main(int argc, char *argv[])
 	if (sock >= 0) close(sock);
 	if (unlink(path) == -1) {
 		syslog(LOG_WARNING, "Failed to unlink socket %s (%s)",
-			path, strerror(errno));
+		       path, strerror(errno));
 	}
+	q_close(queue);
 
 	return 0;
 }
 
-
