@@ -56,7 +56,7 @@ volatile int hup = 0;
 volatile int dump_state = 0;
 static auparse_state_t *au = NULL;
 #define NO_ACTIONS (!hup && !stop && !dump_state)
-#define STATE_FILE "/var/run/ids-state"
+#define STATE_FILE AUDIT_RUN_DIR"/ids-state"
 #define TIMER_INTERVAL 30	// Run every 30 seconds
 struct ids_conf config;
 
@@ -76,7 +76,7 @@ void my_printf(const char *fmt, ...)
 		fputc('\n', stderr);
 	} else if (mode == 3) {
 		if (l == NULL) {
-			l = fopen("/var/run/audisp-ids.log", "wt");
+			l = fopen(AUDIT_RUN_DIR"/audisp-ids.log", "w");
 			if (l == NULL) {
 				va_end(ap);
 				return;
