@@ -58,6 +58,9 @@ _S(NORM_FILE_STAT, "fstat")
 _S(NORM_FILE_STAT, "lstat")
 _S(NORM_FILE_STAT, "stat64")
 _S(NORM_FILE_STAT, "statx")
+_S(NORM_FILE_STAT, "getxattrat")
+_S(NORM_FILE_STAT, "listxattrat")
+_S(NORM_FILE_STAT, "file_getattr")
 _S(NORM_FILE_SYS_STAT, "statfs")
 _S(NORM_FILE_SYS_STAT, "fstatfs")
 _S(NORM_FILE_SYS_STAT, "statmount")
@@ -74,6 +77,7 @@ _S(NORM_FILE, "readlink")
 _S(NORM_FILE, "readlinkat")
 _S(NORM_FILE, "open_by_handle_at")
 _S(NORM_FILE, "pidfd_getfd")
+_S(NORM_FILE, "open_tree_attr")
 _S(NORM_FILE_CHATTR, "removexattr")
 _S(NORM_FILE_CHATTR, "removexattrat")
 _S(NORM_FILE_CHATTR, "fremovexattr")
@@ -126,7 +130,6 @@ _S(NORM_SYSTEM_NAME, "sethostname")
 _S(NORM_SYSTEM_NAME, "setdomainname")
 _S(NORM_SYSTEM_MEMORY, "mmap")
 _S(NORM_SYSTEM_MEMORY, "brk")
-_S(NORM_SYSTEM_MEMORY, "map_shadow_stack")
 _S(NORM_SCHEDULER, "sched_setparam")
 _S(NORM_SCHEDULER, "sched_setscheduler")
 _S(NORM_SCHEDULER, "sched_setattr")
@@ -134,5 +137,7 @@ _S(NORM_SECURITY_POLICY, "landlock_create_ruleset")
 _S(NORM_SECURITY_POLICY, "landlock_add_rule")
 _S(NORM_SECURITY_POLICY, "landlock_restrict_self")
 _S(NORM_SECURITY_POLICY, "lsm_set_self_attr")
+_S(NORM_SECURITY_ATTR, "lsm_get_self_attr")
+_S(NORM_SECURITY_LIST, "lsm_list_modules")
 _S(NORM_SECURITY_POLICY, "mseal")
 
