@@ -64,6 +64,10 @@ char *audit_strsplit_r(char *s, char **savedpp);
 char *audit_strsplit(char *s);
 int audit_is_last_record(int type);
 
+extern const char *SINGLE;
+extern const char *HALT;
+void change_runlevel(const char *level);
+const char *get_progname(void);
 
 #define MINUTES 60
 #define HOURS   60*MINUTES
@@ -90,8 +94,9 @@ void wall_message(const char *fmt, ...)
 
 typedef enum { MSG_STDERR, MSG_SYSLOG, MSG_QUIET } message_t;
 typedef enum { DBG_NO, DBG_YES } debug_message_t;
-void set_aumessage_mode(message_t mode, debug_message_t debug);
+void _set_aumessage_mode(message_t mode, debug_message_t debug);
 
 AUDIT_HIDDEN_END
+
 #endif
 
