@@ -119,21 +119,6 @@ static int audit_priority(int xerrno)
 		return LOG_WARNING;
 }
 
-static const char *get_progname(void)
-{
-	static char progname[256];
-	if (progname[0] == 0) {
-		char tname[256];
-		ssize_t len = readlink("/proc/self/exe",tname,sizeof(tname)-1);
-		if (len != -1) {
-			tname[len] = '\0';
-			strcpy(progname, basename(progname));
-		} else
-			strcpy(progname, "unknown");
-	}
-	return progname;
-}
-
 int audit_request_status(int fd)
 {
 	int rc = audit_send(fd, AUDIT_GET, NULL, 0);
@@ -2226,3 +2211,11 @@ int audit_can_read(void)
 #endif
 }
 
+// Leave this exposed until 2027, then hide it for good.
+// This allows programs already compiled to keep working.
+// But new programs can't see it.
+void set_aumessage_mode(message_t mode, debug_message_t debug)
+{
+	_set_aumessage_mode(mode, debug);
+}
+
