@@ -278,6 +278,7 @@ static int audit_setup_watch_name(struct audit_rule_data **rulep, char *path)
 	int type = AUDIT_WATCH;
 	size_t len;
 	struct stat buf;
+	unsigned int i;
 
 	if (check_path(path))
 		return -1;
@@ -294,13 +295,28 @@ static int audit_setup_watch_name(struct audit_rule_data **rulep, char *path)
 		if (S_ISDIR(buf.st_mode))
 			type = AUDIT_DIR;
 	}
-	/* FIXME: might want to check to see that rule is empty */
-	if (audit_add_watch_dir(type, rulep, path)) 
+	/* Ensure the rule is empty before adding a watch */
+	if ((*rulep)->field_count || (*rulep)->action || (*rulep)->flags ||
+	    (*rulep)->buflen)
+		goto err;
+	for (i = 0; i < AUDIT_MAX_FIELDS; i++)
+		if ((*rulep)->fields[i] || (*rulep)->values[i] ||
+		    (*rulep)->fieldflags[i])
+			goto err;
+	for (i = 0; i < AUDIT_BITMASK_SIZE; i++)
+		if ((*rulep)->mask[i])
+			goto err;
+	if (audit_add_watch_dir(type, rulep, path))
 		return -1;
 
 	if (add != AUDIT_FILTER_UNSET)
 		audit_msg(LOG_INFO, "Old style watch rules are slower");
 	return 1;
+err:
+	audit_msg(LOG_ERR, "Watches may not include fields or actions");
+	audit_rule_free_data(*rulep);
+	*rulep = audit_rule_create_data();
+	return -1;
 }
 
 /*
@@ -960,6 +976,32 @@ static int opt_syscall(opt_handler_params_t *args)
 	return retval;
 }
 
+/*
+ * process_key_option - append key string while enforcing limits
+ * @optarg: key string to append
+ * @key: destination key buffer
+ * @keylen: remaining buffer length
+ * Returns 0 on success or OPT_ERROR_NO_REPLY on error.
+ */
+static int process_key_option(const char *optarg, char *key,
+			      unsigned int *keylen)
+{
+	if ((strlen(optarg) + strlen(key) + (!!key[0])) >
+			AUDIT_MAX_KEY_LEN) {
+		audit_msg(LOG_ERR, "key option exceeds size limit");
+		return OPT_ERROR_NO_REPLY;
+	}
+	if (strchr(optarg, AUDIT_KEY_SEPARATOR))
+		audit_msg(LOG_ERR, "key %s has illegal character", optarg);
+	if (key[0]) {
+		strcat(key, key_sep);
+		(*keylen)--;
+	}
+	strncat(key, optarg, *keylen);
+	*keylen = AUDIT_MAX_KEY_LEN - strlen(key);
+	return 0;
+}
+
 static int opt_field(opt_handler_params_t *args)
 {
 	int retval = args->retval, rc;
@@ -978,22 +1020,9 @@ static int opt_field(opt_handler_params_t *args)
 	// Keys need to get handled differently
 	if (strncmp(optarg, "key=", 4) == 0) {
 		optarg += 4;
-		// goto process_keys;
-		if ((strlen(optarg)+strlen(key)+(!!key[0])) >
-							AUDIT_MAX_KEY_LEN) {
-			audit_msg(LOG_ERR, "key option exceeds size limit");
-			retval = OPT_ERROR_NO_REPLY;
-		} else {
-			if (strchr(optarg, AUDIT_KEY_SEPARATOR))
-				audit_msg(LOG_ERR,
-				    "key %s has illegal character", optarg);
-			if (key[0]) { // Add the separator if we need to
-				strcat(key, key_sep);
-				keylen--;
-			}
-			strncat(key, optarg, keylen);
-			keylen = AUDIT_MAX_KEY_LEN - strlen(key);
-		}
+		rc = process_key_option(optarg, key, &keylen);
+		if (rc)
+			retval = rc;
 		return retval;
 	}
 
@@ -1140,7 +1169,7 @@ static int opt_remove_watch(opt_handler_params_t *args)
 
 static int opt_key(opt_handler_params_t *args)
 {
-	int retval = args->retval;
+	int rc, retval = args->retval;
 	if (!(_audit_syscalladded || _audit_permadded ||
 		_audit_exeadded || _audit_filterfsadded) ||
 		(add == AUDIT_FILTER_UNSET && del == AUDIT_FILTER_UNSET)) {
@@ -1152,24 +1181,9 @@ static int opt_key(opt_handler_params_t *args)
 		return OPT_ERROR_NO_REPLY;
 	}
 
-	// FIXME refactor this to a function
-	// process_keys:
-	if ((strlen(optarg) + strlen(key) + (!!key[0])) >
-		AUDIT_MAX_KEY_LEN) {
-		audit_msg(LOG_ERR, "key option exceeds size limit");
-		retval = -1;
-	} else {
-		if (strchr(optarg, AUDIT_KEY_SEPARATOR))
-			audit_msg(LOG_ERR,
-					  "key %s has illegal character",
-					  optarg);
-		if (key[0]) { // Add the separator if we need to
-			strcat(key, key_sep);
-			keylen--;
-		}
-		strncat(key, optarg, keylen);
-		keylen = AUDIT_MAX_KEY_LEN - strlen(key);
-	}
+	rc = process_key_option(optarg, key, &keylen);
+	if (rc)
+		retval = rc;
 	return retval;
 }
 
@@ -1676,7 +1690,7 @@ int main(int argc, char *argv[])
 {
 	int retval = 1;
 
-	set_aumessage_mode(MSG_STDERR, DBG_NO);
+	_set_aumessage_mode(MSG_STDERR, DBG_NO);
 
 	if (argc == 1) {
 		usage();
@@ -1696,7 +1710,7 @@ int main(int argc, char *argv[])
 	if ((argc == 3) && (strcmp(argv[1], "-R") == 0)) {
 		// If reading a file, its most likely start up. Send problems
 		// to syslog where they will persist for later review
-		set_aumessage_mode(MSG_SYSLOG, DBG_NO);
+		_set_aumessage_mode(MSG_SYSLOG, DBG_NO);
 		fd = audit_open();
 		if (is_ready() == 0)
 			return 1;
@@ -1770,9 +1784,9 @@ static int handle_request(int status)
 					audit_rule_syscallbyname_data(
 							rule_new, "all");
 			}
-			set_aumessage_mode(MSG_QUIET, DBG_NO);
+			_set_aumessage_mode(MSG_QUIET, DBG_NO);
 			rc = audit_add_rule_data(fd, rule_new, add, action);
-			set_aumessage_mode(MSG_STDERR, DBG_NO);
+			_set_aumessage_mode(MSG_STDERR, DBG_NO);
 			/* Retry for legacy kernels */
 			if (rc < 0) {
 				if (errno == EINVAL &&
@@ -1794,10 +1808,10 @@ static int handle_request(int status)
 					audit_rule_syscallbyname_data(
 							rule_new, "all");
 			}
-			set_aumessage_mode(MSG_QUIET, DBG_NO);
+			_set_aumessage_mode(MSG_QUIET, DBG_NO);
 			rc = audit_delete_rule_data(fd, rule_new,
 								 del, action);
-			set_aumessage_mode(MSG_STDERR, DBG_NO);
+			_set_aumessage_mode(MSG_STDERR, DBG_NO);
 			/* Retry for legacy kernels */
 			if (rc < 0) {
 				if (errno == EINVAL &&
