@@ -78,8 +78,8 @@ volatile ATOMIC_INT stop = 0;
 /* Local data */
 static int fd = -1, pipefds[2] = {-1, -1};
 static struct daemon_conf config;
-static const char *pidfile = "/var/run/auditd.pid";
-static const char *state_file = "/var/run/auditd.state";
+static const char *pidfile = AUDIT_RUN_DIR"/auditd.pid";
+static const char *state_file = AUDIT_RUN_DIR"/auditd.state";
 static int init_pipe[2];
 static int do_fork = 1, opt_aggregate_only = 0, config_dir_set = 0;
 /*
@@ -104,6 +104,7 @@ static char *getsubj(char *subj);
 /* Manage access to the preallocated event pool */
 static struct auditd_event *alloc_pool_event(void);
 int event_is_prealloc(struct auditd_event *e);
+static int make_audit_run_dir(void);
 
 enum startup_state {startup_disable=0, startup_enable, startup_nochange,
 	startup_INVALID};
@@ -112,6 +113,7 @@ static const char *startup_states[] = {"disable", "enable", "nochange"};
 /*
  * Output a usage message
  */
+static void usage(void) __attribute__((noreturn));
 static void usage(void)
 {
 	fprintf(stderr,
@@ -402,6 +404,24 @@ int send_audit_event(int type, const char *str)
 	return 0;
 }
 
+
+static int make_audit_run_dir(void)
+{
+	struct stat st;
+
+	if (stat(AUDIT_RUN_DIR, &st) < 0) {
+		if (mkdir(AUDIT_RUN_DIR, 0755) < 0) {
+			audit_msg(LOG_ERR,
+				"Cannot create run directory %s (%s)",
+				AUDIT_RUN_DIR, strerror(errno));
+			return -1;
+		}
+	} else if (!S_ISDIR(st.st_mode))
+		return -1;
+
+	return 0;
+}
+
 static int write_pid_file(void)
 {
 	int pidfd, len;
@@ -574,13 +594,12 @@ static void netlink_handler(struct ev_loop *loop, struct ev_io *io,
 {
 	int rc = 1, cnt = 0;
 
-	// Try to get all the events that are waiting but yield after 5 to
-	// let other handlers run. Five should cover PATH events.
-	// FIXME: backing down to 3 until IPC is faster
-	while (rc > 0 && cnt < 3) {
+	// Try to get all the events that are waiting but yield after 4 to
+	// let other handlers run. Four should cover file watch events.
+	while (rc > 0 && cnt < 4) {
 		if (cur_event == NULL) {
 			cur_event = alloc_pool_event();
-				if (cur_event == NULL) {
+			if (cur_event == NULL) {
 				char emsg[DEFAULT_BUF_SZ];
 				if (*subj)
 					snprintf(emsg, sizeof(emsg),
@@ -765,10 +784,10 @@ int main(int argc, char *argv[])
 
 	if (opt_foreground) {
 		config.daemonize = D_FOREGROUND;
-		set_aumessage_mode(MSG_STDERR, DBG_YES);
+		_set_aumessage_mode(MSG_STDERR, DBG_YES);
 	} else {
 		config.daemonize = D_BACKGROUND;
-		set_aumessage_mode(MSG_SYSLOG, DBG_NO);
+		_set_aumessage_mode(MSG_SYSLOG, DBG_NO);
 		(void) umask( umask( 077 ) | 022 );
 	}
 	session = audit_get_session();
@@ -841,13 +860,19 @@ int main(int argc, char *argv[])
 			tell_parent(FAILURE);
 			free_config(&config);
 			return 1;
-		} 
+		}
 		openlog("auditd", LOG_PID, LOG_DAEMON);
 	}
 
 	/* Init netlink */
 	if ((fd = audit_open()) < 0) {
-        	audit_msg(LOG_ERR, "Cannot open netlink audit socket");
+		audit_msg(LOG_ERR, "Cannot open netlink audit socket");
+		tell_parent(FAILURE);
+		free_config(&config);
+		return 1;
+	}
+
+	if (make_audit_run_dir() < 0) {
 		tell_parent(FAILURE);
 		free_config(&config);
 		return 1;
