@@ -34,6 +34,7 @@
 #include <netdb.h>
 #include <limits.h>	/* PATH_MAX */
 #include <ctype.h>
+#include <unistd.h>
 #include "libaudit.h"
 #include "ausearch-options.h"
 #include "ausearch-lookup.h"
@@ -79,7 +80,9 @@ static int audit_avc_init(search_items *s)
 }
 
 /*
- * This function will take the list and extract the searchable fields from it.
+ * This function will take the audit event as a list and extract the
+ * searchable fields from it. It does this by iterating over each record
+ * in the event and branching to the right parser for each record type.
  * It returns 0 on success and 1 on failure.
  */
 int extract_search_items(llist *l)
@@ -1938,8 +1941,11 @@ static int parse_integrity(const lnode *n, search_items *s)
 }
 
 
-/* FIXME: If they are in permissive mode or hit an auditallow, there can 
- * be more than 1 avc in the same syscall. For now, we pickup just the first.
+/*
+ * If they are in permissive mode or hit an auditallow, there can be
+ * more than 1 avc in the same syscall/event. Each one will be it's own
+ * record. We will see each AVC one record at a time. The first one will
+ * initialize the anode list and subsequent ones will just add to it.
  */
 static int parse_avc(const lnode *n, search_items *s)
 {
@@ -1959,12 +1965,12 @@ static int parse_avc(const lnode *n, search_items *s)
 			term = n->message;
 			goto other_avc;
 		}
-		if (event_success != S_UNSET) {
+		// Do not override syscall success if already set.
+		// Syscall pass/fail is the authoritative value.
+		if (event_success != S_UNSET && s->success == S_UNSET) {
 			*term = 0;
-			// FIXME. Do not override syscall success if already
-			// set. Syscall pass/fail is the authoritative value.
 			if (strstr(str, "denied")) {
-				s->success = S_FAILED; 
+				s->success = S_FAILED;
 				an.avc_result = AVC_DENIED;
 			} else {
 				s->success = S_SUCCESS;
@@ -2041,7 +2047,7 @@ other_avc:
 			*term = 0;
 			s->comm = strdup(str);
 			*term = '"';
-		} else { 
+		} else {
 			s->comm = unescape(str);
 			if (s->comm == NULL) {
 				rc = 11;
@@ -2116,6 +2122,7 @@ other_avc:
 	if (term)
 		*term = ' ';
 
+	// This can be called multiple times. Only first time it initializes.
 	if (audit_avc_init(s) == 0) {
 		alist_append(s->avc, &an);
 	} else {
@@ -2824,3 +2831,133 @@ static int parse_kernel(lnode *n, search_items *s)
 	return 0;
 }
 
+//////// These are used by ausearch / aureport ///////////////
+
+/* read_first_ts - get timestamp of the first record in a log file
+ * @file: path to the log file
+ * @sec: returned seconds component
+ * @milli: returned milliseconds component
+ *
+ * Returns 0 on success and -1 on failure.
+ */
+static int read_first_ts(const char *file, time_t *sec, unsigned int *milli)
+{
+	FILE *fp;
+	char buf[MAX_AUDIT_MESSAGE_LENGTH];
+	char *p, *e;
+
+	*sec = 0;
+	*milli = 0;
+
+	fp = fopen(file, "rm");
+	if (fp == NULL)
+		return -1;
+	if (fgets_unlocked(buf, sizeof(buf), fp) == NULL) {
+		fclose(fp);
+		return -1;
+	}
+	fclose(fp);
+
+	p = strstr(buf, "audit(");
+	if (p == NULL)
+		return -1;
+	p += 6;
+	*sec = strtoll(p, &e, 10);
+	if (*e == '.')
+		*milli = strtoul(e + 1, &e, 10);
+
+	return 0;
+}
+
+/* audit_log_list - collect audit logs and their first timestamps
+ * @basefile: base audit log file
+ * @logs: returned array of log information
+ * @log_cnt: number of entries in @logs
+ *
+ * Returns 0 on success and -1 on failure.
+ */
+int audit_log_list(const char *basefile, struct audit_log_info **logs,
+                        size_t *log_cnt)
+{
+	char *filename;
+	size_t len;
+	int num = 0;
+	struct audit_log_info *list = NULL;
+
+	len = strlen(basefile) + 16;
+	filename = malloc(len);
+	if (filename == NULL)
+		return -1;
+
+	snprintf(filename, len, "%s", basefile);
+	do {
+		struct audit_log_info *tmp;
+		time_t sec;
+		unsigned int milli;
+
+		if (access(filename, R_OK) != 0)
+			break;
+		if (read_first_ts(filename, &sec, &milli))
+			sec = 0;
+		tmp = realloc(list, (num + 1) * sizeof(*list));
+		if (tmp == NULL) {
+			free(filename);
+			audit_log_free(list, num);
+			return -1;
+		}
+		list = tmp;
+		list[num].name = strdup(filename);
+		if (list[num].name == NULL) {
+			free(filename);
+			audit_log_free(list, num);
+			return -1;
+		}
+		list[num].sec = sec;
+		list[num].milli = milli;
+		num++;
+		snprintf(filename, len, "%s.%d", basefile, num);
+	} while (1);
+
+	free(filename);
+	*logs = list;
+	*log_cnt = num;
+	return 0;
+}
+
+/* audit_log_find_start - choose oldest log that may contain @start
+ * @logs: array of log information
+ * @log_cnt: number of logs
+ * @start: requested start time
+ *
+ * Returns index of the log to begin processing from.
+ */
+unsigned audit_log_find_start(const struct audit_log_info *logs,
+			      size_t log_cnt, time_t start)
+{
+	size_t start_idx = log_cnt ? log_cnt - 1 : 0;
+
+	if (start) {
+		ssize_t i;
+		for (i = log_cnt - 1; i >= 0; i--) {
+			if (logs[i].sec > start) {
+				if ((size_t)i < log_cnt - 1)
+					start_idx = i + 1;
+				break;
+			}
+		}
+	}
+	return start_idx;
+}
+
+/* audit_log_free - release memory held by audit_log_list */
+void audit_log_free(struct audit_log_info *logs, size_t log_cnt)
+{
+	size_t i;
+
+	if (logs == NULL)
+		return;
+	for (i = 0; i < log_cnt; i++)
+		free(logs[i].name);
+	free(logs);
+}
+
